﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Text;

namespace WpfAppControl
{
    /// <summary>
    /// </summary>
    public class SocketMessageEventArgs : EventArgs
    {
        /// <summary>
        /// </summary>
        public string Message { set; get; }
    }

    /// <summary>
    /// </summary>
    public class SocketChatClient
    {
        private readonly object _locker = new object();

        private readonly TcpClient _client;

        /// <summary>
        /// </summary>
        public string ClientIP { set; get; }

        /// <summary>
        /// </summary>
        public Encoding CurrentEncoding { set; get; } = Encoding.UTF8;

        private readonly TcpListener _listener;

        private readonly byte[] data;

        /// <summary>
        /// </summary>
        public EventHandler<SocketMessageEventArgs> SocketMessageHandler { set; get; }

        /// <summary>
        /// </summary>
        /// <param name="ipStr"></param>
        /// <param name="prot"></param>
        public SocketChatClient(string ipStr, int prot)
        {
            // 初始化服务器IP
            var localAdd = IPAddress.Parse(ipStr);

            // 创建TCP侦听器
            _listener = new TcpListener(localAdd, prot);

            _listener.Start();

            this._client = _listener.AcceptTcpClient();
            this._client.ReceiveBufferSize = 819200;
            this.ClientIP = ipStr;
            data = new byte[this._client.ReceiveBufferSize];

            RegisterReceiveMessage();
        }

        /// <summary>
        /// </summary>
        /// <param name="client"></param>
        public SocketChatClient(TcpClient client)
        {
            this._client = client;
            this.ClientIP = _client.Client.RemoteEndPoint.ToString();
            data = new byte[this._client.ReceiveBufferSize];

            RegisterReceiveMessage();
        }

        /// <summary>
        /// 从客戶端获取消息
        /// </summary>
        /// <param name="ar"></param>
        private void ReceiveMessage(IAsyncResult ar)
        {
            try
            {
                int bytesRead;
                lock (_locker)
                {
                    bytesRead = this._client.GetStream().EndRead(ar);
                }

                if (bytesRead < 1)
                {
                    return;
                }
                var message = CurrentEncoding.GetString(data, 0, bytesRead);
                SocketMessageHandler?.Invoke(this, new SocketMessageEventArgs { Message = message });
                RegisterReceiveMessage();
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }
        }

        /// <summary>
        /// 向客戶端发送消息
        /// </summary>
        /// <param name="message"></param>
        public void SendMessage(string message)
        {
            try
            {
                NetworkStream ns = null;
                lock (_locker)
                {
                    ns = this._client.GetStream();
                }
                // 对信息进行编码
                var bytesToSend = CurrentEncoding.GetBytes(message);
                ns.Write(bytesToSend, 0, bytesToSend.Length);
                ns.Flush();
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }
        }

        private void RegisterReceiveMessage()
        {
            lock (_locker)
            {
                this._client.GetStream().BeginRead(data, 0, Convert.ToInt32(this._client.ReceiveBufferSize), ReceiveMessage, null);
            }
        }

        /// <summary>
        /// </summary>
        public void Stop()
        {
            try
            {
                if (_client != null)
                    _client.Close();
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }
            try
            {
                if (_listener != null)
                    _listener.Stop();
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }
        }
    }
}